package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.service.FactoryService;
import com.abi.task.common.api.base.BaseResponse;
import com.abi.task.common.excel.ExcelInputFactory;
import com.abi.task.common.excel.preperties.FactoryProperty;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 供应商管理
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@RequestMapping("factory")
@Api(tags = "供应商管理")
public class FactoryController {

    @Autowired
    private FactoryService factoryService;

    @ApiOperation("供应商新增修改")
    @PostMapping("saveFactory")
    public BaseResponse saveFactory(@Validated @RequestBody SaveFactoryReq saveFactoryReq) throws Exception {
        //供应商新增修改
        factoryService.saveFactory(saveFactoryReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("供应商查询分页")
    @GetMapping("listFactory")
    public BaseResponse<ListFactoryRes> listFactory(@Validated ListFactoryReq listFactoryReq) throws Exception {
        //供应商查询分页
        ListFactoryRes result = factoryService.listFactory(listFactoryReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("供应商查询详情")
    @GetMapping("getFactoryDetail")
    public BaseResponse<GetFactoryDetailRes> getFactoryDetail(@Validated GetFactoryDetailReq getFactoryDetailReq) throws Exception {
        //供应商查询详情
        GetFactoryDetailRes result = factoryService.getFactoryDetail(getFactoryDetailReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("供应商启用")
    @PostMapping("enableFactory")
    public BaseResponse enableFactory(@Validated @RequestBody EnableFactoryReq enableFactoryReq) throws Exception {
        //供应商启用
        factoryService.enableFactory(enableFactoryReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("供应商禁用")
    @PostMapping("disableFactory")
    public BaseResponse disableFactory(@Validated @RequestBody DisableFactoryReq disableFactoryReq) throws Exception {
        //供应商禁用
        factoryService.disableFactory(disableFactoryReq);
        //包装出参
        return BaseResponse.create();
    }

    @ResponseBody
    @ApiOperation(value = "导出模板")
    @GetMapping("downloadExcelModel")
    public void downloadExcelModel(HttpServletResponse response) throws Exception {
        ExcelInputFactory.downloadExcelModel("供应商模板", response, FactoryProperty.class);
    }

    @ApiOperation("excel批量导入")
    @PostMapping("factoryExcelImport")
    public BaseResponse factoryExcelImport(@RequestParam("file") MultipartFile file) throws Exception {
        List<FactoryProperty> factoryPropertyList = ExcelInputFactory.getExcelPojoList(file, FactoryProperty.class);
        //excel批量导入
        ExcelImportRes result = factoryService.factoryExcelImport(factoryPropertyList);
        //包装出参
        return BaseResponse.create(result);

    }

    @ApiOperation("启用中的供应商列表")
    @GetMapping("listFactoryEnable")
    public BaseResponse<ListFactoryEnableRes> listFactoryEnable() throws Exception {
        ListFactoryEnableRes result = factoryService.listFactoryEnable();
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("供应商下拉框(包材厂，二级包材厂，啤酒厂)")
    @PostMapping("listFactorySelectRes")
    public BaseResponse<ListFactorySelectRes> listFactorySelectRes(@RequestBody @Validated ListFactorySelectReq req) {
        //包材厂下拉框
        ListFactorySelectRes result = factoryService.listFactorySelectRes(req);
        //包装出参
        return BaseResponse.create(result);
    }


    //    6.03已废弃
//    @ApiOperation("啤酒厂下拉框")
//    @GetMapping("listBeerFactorySelect")
//    public BaseResponse<ListBeerFactorySelectRes> listBeerFactorySelect() {
//        //啤酒厂下拉框
//        ListBeerFactorySelectRes result = factoryService.listBeerFactorySelect();
//        //包装出参
//        return BaseResponse.create(result);
//    }
//
//    @ApiOperation("包材厂下拉框")
//    @GetMapping("listCoverFactorySelect")
//    public BaseResponse<ListCoverFactorySelectRes> listCoverFactorySelect() {
//        //包材厂下拉框
//        ListCoverFactorySelectRes result = factoryService.listCoverFactorySelect();
//        //包装出参
//        return BaseResponse.create(result);
//    }

}








