package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.annotation.PassToken;
import com.abi.qms.platform.dto.req.LoginReq;
import com.abi.qms.platform.dto.req.ResetPasswordReq;
import com.abi.qms.platform.dto.req.UpdatePasswordReq;
import com.abi.qms.platform.dto.res.LoginRes;
import com.abi.qms.platform.service.LoginService;
import com.abi.task.common.annotation.IgnoreToken;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
 * 登录
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@RequestMapping("login")
@Api(tags = "登录")
public class LoginController {

    @Autowired
    private LoginService loginService;

    /**
     * 登录
     *
     * @param req
     * @return
     */
    @PostMapping("login")
    @ApiOperation("登录")
    @PassToken
    public BaseResponse loginQms(@Validated @RequestBody LoginReq req) {
        LoginRes login = loginService.login(req);
        return BaseResponse.create(login);
    }


    @ApiOperation("重置密码")
    @PostMapping("resetPassword")
    public BaseResponse resetPassword(@Validated @RequestBody ResetPasswordReq req) throws Exception {
        loginService.resetPassword(req);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("修改密码")
    @PostMapping("updatePassword")
    public BaseResponse updatePassword(@Validated UpdatePasswordReq req) throws Exception {
        loginService.updatePassword(req);
        return BaseResponse.create();
    }
}








