package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.ActiveBoxCodeReq;
import com.abi.qms.platform.dto.req.GenerateBarCodeReq;
import com.abi.qms.platform.dto.req.ListQrBoxCodeMappingReq;
import com.abi.qms.platform.dto.req.PrintingDetailReq;
import com.abi.qms.platform.dto.res.ListQrBoxCodeMappingRes;
import com.abi.qms.platform.dto.res.PrintingDetailRes;
import com.abi.qms.platform.dto.res.QrBoxCodeUploadRes;
import com.abi.qms.platform.service.QrBoxMappingService;
import com.abi.task.common.api.base.BaseResponse;
import com.abi.task.common.excel.ExcelInputFactory;
import com.abi.task.common.excel.preperties.QrBoxMappingProperty;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

/**
 * @author: fangxinjian
 * @date: 2021/05/27 11:15
 * @description: 箱码合一
 */
@RestController
@Api(tags = "箱码合一")
@RequestMapping("/boxCodeUnity")
public class QrBoxCodeUnityController {

    @Autowired
    private QrBoxMappingService qrBoxMappingService;

    @ApiOperation("生成条形码")
    @PostMapping("/generateBarCode")
    public BaseResponse generateBarCode(@RequestBody GenerateBarCodeReq generateBarCodeReq) {

        qrBoxMappingService.generateBarCode(generateBarCodeReq);
        return BaseResponse.create();
    }

    @ApiOperation("箱码合一文件上传")
    @PostMapping("/uploads/{packageId}")
    public BaseResponse<QrBoxCodeUploadRes> uploadCodeFiles(HttpServletRequest request, @PathVariable("packageId") Long packageId) {

        //拿到文件
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> map = multipartRequest.getFileMap();

        QrBoxCodeUploadRes res = qrBoxMappingService.uploadCodeFiles(map, packageId);

        return BaseResponse.create(res);

    }

    @ApiOperation("分页查询箱码合一列表")
    @GetMapping("/listQrBoxCodeMapping")
    public BaseResponse<ListQrBoxCodeMappingRes> listQrBoxCodeMapping(ListQrBoxCodeMappingReq listQrBoxCodeMappingReq) {

        ListQrBoxCodeMappingRes res = qrBoxMappingService.listQrBoxCodeMapping(listQrBoxCodeMappingReq);
        return BaseResponse.create(res);
    }

    @ApiOperation("激活箱码")
    @PostMapping("/active")
    public BaseResponse activeBoxCode(@RequestBody ActiveBoxCodeReq activeBoxCodeReq) {

        qrBoxMappingService.activeBoxCode(activeBoxCodeReq);
        return BaseResponse.create();
    }

    @ApiOperation("作废箱码")
    @PostMapping("/invalid")
    public BaseResponse invalidBoxCode(@RequestBody ActiveBoxCodeReq activeBoxCodeReq) {

        qrBoxMappingService.invalidBoxCode(activeBoxCodeReq);
        return BaseResponse.create();
    }

    @ApiOperation("打印")
    @GetMapping("/printingDetail")
    public BaseResponse<PrintingDetailRes> getPrintingDetail(PrintingDetailReq printingDetailReq) {

        PrintingDetailRes res = qrBoxMappingService.getPrintingDetail(printingDetailReq);
        return BaseResponse.create(res);
    }

    @ResponseBody
    @ApiOperation(value = "导出模板")
    @GetMapping("downloadExcelModel")
    public void downloadExcelModel(HttpServletResponse response) throws Exception {
        ExcelInputFactory.downloadExcelModel("箱码合一", response, QrBoxMappingProperty.class);
    }


}
