package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.AssociateSapOrderReq;
import com.abi.qms.platform.dto.req.QueryOrderFormDetailReq;
import com.abi.qms.platform.dto.req.QueryOrderFormLimitReq;
import com.abi.qms.platform.dto.req.SaveOrderFormReq;
import com.abi.qms.platform.dto.res.QueryOrderFormDetailRes;
import com.abi.qms.platform.dto.res.QueryOrderFormLimitRes;
import com.abi.qms.platform.dto.res.ReservationOrderQueryRes;
import com.abi.qms.platform.service.QrPackageBookingOrderService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author ludashi
 * @date 2021年06月04日 15:02
 * 预订单管理
 */
@Slf4j
@RestController
@Api(tags = "预订单管理")
@RequestMapping("/orderForm")
public class QrPackageBookingOrderController {

    @Autowired
    private QrPackageBookingOrderService qrPackageBookingOrderService;

    @ApiOperation("新增修改预订单")
    @PostMapping("/saveOrderForm")
    public BaseResponse saveOrderForm(@Validated @RequestBody SaveOrderFormReq saveOrderFormReq) {
        qrPackageBookingOrderService.saveOrderForm(saveOrderFormReq);
       return BaseResponse.create();
    }

    @ApiOperation("查看预订单详情")
    @GetMapping("/queryOrderFormDetail")
    public BaseResponse<QueryOrderFormDetailRes> queryOrderFormDetail(@Validated QueryOrderFormDetailReq queryOrderFormDetailReq) {
        QueryOrderFormDetailRes queryOrderFormDetailRes = qrPackageBookingOrderService.queryOrderFormDetail(queryOrderFormDetailReq);
        return BaseResponse.create(queryOrderFormDetailRes);
    }

    @ApiOperation("查看预订单查询分页")
    @GetMapping("/queryOrderFormLimit")
    public BaseResponse<QueryOrderFormLimitRes> queryOrderFormLimit(@Validated QueryOrderFormLimitReq queryOrderFormLimitReq) {
        QueryOrderFormLimitRes res=qrPackageBookingOrderService.queryOrderFormLimit(queryOrderFormLimitReq);
        return BaseResponse.create(res);
    }

    @ApiOperation("关联单号")
    @PostMapping("/associateSapOrder")
    public BaseResponse associateSapOrder(@Validated @RequestBody AssociateSapOrderReq associateSapOrderReq) {
        qrPackageBookingOrderService.associateSapOrder(associateSapOrderReq);
        return BaseResponse.create();
    }

    @ApiOperation("预订单下拉框选项")
    @GetMapping("/reservationOrderQuery")
    public BaseResponse<ReservationOrderQueryRes> reservationOrderQuery() {
        ReservationOrderQueryRes res = qrPackageBookingOrderService.reservationOrderQuery();
        return BaseResponse.create(res);
    }
}
