package com.abi.qms.platform.infrastructure.util;

import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.dao.mapper.UserInfoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 用户工具类
 *
 * @author AndyTan
 */
@Component
public class UserUtil {

    @Autowired
    private UserInfoMapper userInfoMapper;

    /**
     * 仅token中的user信息
     */
    private final InheritableThreadLocal<UserInfo> USER_THREAD_LOCAL = new InheritableThreadLocal();

    public void setUser(UserInfo userInfo) {
        USER_THREAD_LOCAL.set(userInfo);
    }

    public UserInfo getUser() {
        return USER_THREAD_LOCAL.get();
    }

    public void releaseUser() {
        USER_THREAD_LOCAL.remove();
    }

    /**
     * 查询数据库后完整的user信息
     */
    private final InheritableThreadLocal<UserInfo> USER_WHOLE_THREAD_LOCAL = new InheritableThreadLocal();

    private synchronized void initWholeUser() {
        if(USER_WHOLE_THREAD_LOCAL.get()!=null){
            return;
        }
        UserInfo wholeUser = userInfoMapper.selectById(USER_THREAD_LOCAL.get().getId());
        USER_WHOLE_THREAD_LOCAL.set(wholeUser);
    }

    public UserInfo getWholeUser() {
        if(USER_WHOLE_THREAD_LOCAL.get()==null && USER_THREAD_LOCAL.get()!=null && USER_THREAD_LOCAL.get().getId()!=null){
            initWholeUser();
        }
        return USER_WHOLE_THREAD_LOCAL.get();
    }

}
