package com.abi.qms.platform.service;


import com.abi.qms.platform.dao.entity.UserRole;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;

/**
 * <p>
 * 角色管理
 * </p>
 *
 * @author Andy.Tan
 */
public interface RoleManagerService {

    /**
     * 创建角色
     *
     * @param createRoleReq
     * @throws Exception
     */
    void createRole(CreateRoleReq createRoleReq) throws Exception;

    /**
     * 修改角色
     * @param updateRoleReq
     * @throws Exception
     */
    void updateRole(UpdateRoleReq updateRoleReq) throws Exception;

    /**
     * 查询单个角色信息
     * @param getRoleInfoReq
     * @return
     * @throws Exception
     */
    GetRoleInfoRes getRoleInfo(GetRoleInfoReq getRoleInfoReq) throws Exception;

    /**
     * 查询角色列表
     * @param listRoleReq
     * @return
     * @throws Exception
     */
    ListRoleRes listRole(ListRoleReq listRoleReq) throws Exception;

    /**
     * 角色code查询角色信息
     *
     * @param roleCode
     * @return
     */
    UserRole getRoleByCode(String roleCode);

    /**
     * 查询当前登陆用户角色的resource树
     * @param listResourceTreeByRoleCodeReq
     * @return
     * @throws Exception
     */
    ListResourceTreeByRoleCodeRes listResourceTreeByRoleCode(ListResourceTreeByRoleCodeReq listResourceTreeByRoleCodeReq) throws Exception;

    /**
     * 获取选择的上级角色对应的菜单/操作权限树
     * @param listResourceTreeBySelectRoleCodeReq
     * @return
     * @throws Exception
     */
    ListResourceTreeBySelectRoleCodeRes listResourceTreeBySelectRoleCode(ListResourceTreeBySelectRoleCodeReq listResourceTreeBySelectRoleCodeReq) throws Exception;

    /**
     * 获取登录用户需要的页面以及按钮权限
     * @return
     * @throws Exception
     */
    ListLoginUserResourceRes listLoginUserResource() throws Exception;

    /**
     * 启用角色
     * @param enableRoleReq
     * @throws Exception
     */
    void enableRole(EnableRoleReq enableRoleReq) throws Exception;

    /**
     * 禁用角色
     * @param disableRoleReq
     * @throws Exception
     */
    void disableRole(DisableRoleReq disableRoleReq) throws Exception;




}
