package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.BaseSapBeerFactory;
import com.abi.qms.platform.dao.entity.BaseSapCoverFactory;
import com.abi.qms.platform.dao.mapper.BaseSapBeerFactoryMapper;
import com.abi.qms.platform.dto.req.BatchImportSapBeerFactoryReq;
import com.abi.qms.platform.service.BaseSapBeerFactoryService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/04 17:14
 * @description:
 */
@Service
public class BaseSapBeerFactoryServiceImpl implements BaseSapBeerFactoryService {

    @Autowired
    private BaseSapBeerFactoryMapper baseSapBeerFactoryMapper;

    @Override
    public void batchImportSapBeerFactory(BatchImportSapBeerFactoryReq req) {

        List<BatchImportSapBeerFactoryReq.BatchImportBeerFactory> beerFactoryList = req.getBatchImportBeerFactoryList();
        for (BatchImportSapBeerFactoryReq.BatchImportBeerFactory beerFactory : beerFactoryList) {

            BaseSapBeerFactory baseSapBeerFactory = new BaseSapBeerFactory();
            baseSapBeerFactory.setSapBeerFactoryId(beerFactory.getSapBeerFactoryId());
            baseSapBeerFactory.setSapBeerFactoryName(beerFactory.getSapBeerFactoryName());

            saveBaseSapBeerFactory(baseSapBeerFactory);
        }

    }


    @Override
    public void batchUpdateSapBeerFactory(BatchImportSapBeerFactoryReq req) {

        List<BatchImportSapBeerFactoryReq.BatchImportBeerFactory> beerFactoryList = req.getBatchImportBeerFactoryList();
        for (BatchImportSapBeerFactoryReq.BatchImportBeerFactory beerFactory : beerFactoryList) {

            BaseSapBeerFactory baseSapBeerFactory = new BaseSapBeerFactory();
            baseSapBeerFactory.setSapBeerFactoryId(beerFactory.getSapBeerFactoryId());
            baseSapBeerFactory.setSapBeerFactoryName(beerFactory.getSapBeerFactoryName());

            updateBaseSapBeerFactory(baseSapBeerFactory);
        }

    }

    private void updateBaseSapBeerFactory(BaseSapBeerFactory sapBeerFactory) {

        //根据sap啤酒厂ID查询啤酒厂信息
        BaseSapBeerFactory baseSapBeerFactory = getBaseSapCoverFactoryInfo(sapBeerFactory.getSapBeerFactoryId());

        if (ObjectUtil.isNull(baseSapBeerFactory)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap啤酒厂ID不存在");
        }

        sapBeerFactory.setId(baseSapBeerFactory.getId());
        sapBeerFactory.setUpdateTime(LocalDateTime.now());
        baseSapBeerFactoryMapper.updateById(sapBeerFactory);
    }

    private void saveBaseSapBeerFactory(BaseSapBeerFactory sapBeerFactory) {
        //根据sap啤酒厂ID查询啤酒厂信息
        BaseSapBeerFactory baseSapBeerFactory = getBaseSapCoverFactoryInfo(sapBeerFactory.getSapBeerFactoryId());

        if (ObjectUtil.isNotNull(baseSapBeerFactory)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap啤酒厂ID已存在");
        }

        baseSapBeerFactoryMapper.insert(sapBeerFactory);

    }


    /**
     * 根据sap啤酒厂ID查询sap啤酒厂信息
     *
     * @param sapBeerFactoryId sap啤酒厂ID
     * @return
     */
    private BaseSapBeerFactory getBaseSapCoverFactoryInfo(String sapBeerFactoryId) {

        QueryWrapper<BaseSapBeerFactory> qw = new QueryWrapper<>();
        qw.eq("sap_beer_factory_id", sapBeerFactoryId);
        qw.eq("is_delete", 0);

        //根据sap啤酒厂ID查询sap啤酒厂信息
        BaseSapBeerFactory baseSapBeerFactory = baseSapBeerFactoryMapper.selectOne(qw);

        return baseSapBeerFactory;

    }


}
