package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.qms.platform.dao.entity.BaseSapBrand;
import com.abi.qms.platform.dao.mapper.BaseSapBrandMapper;
import com.abi.qms.platform.dto.req.BatchImportBrandReq;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.service.BaseSapBrandService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/02 10:26
 * @description:
 */
@Service
public class BaseSapBrandServiceImpl implements BaseSapBrandService {

    @Autowired
    private BaseSapBrandMapper baseSapBrandMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchImportBrand(BatchImportBrandReq req) {
        List<BatchImportBrandReq.SapBrandBean> brandBeanList = req.getSapBrandBeanList();

        for (BatchImportBrandReq.SapBrandBean brand : brandBeanList) {

            //申明对象
            BaseSapBrand sapBrand = new BaseSapBrand();
            sapBrand.setId(brand.getId());
            sapBrand.setBrandCode(brand.getBrandCode());
            sapBrand.setBrandLevel(brand.getBrandLevel());
            sapBrand.setBrandName(brand.getBrandName());
            sapBrand.setParentBrandCode(brand.getParentBrandCode());
            sapBrand.setParentBrandName(brand.getParentBrandName());


            saveBaseSapBrand(sapBrand);
        }
    }

    public void saveBaseSapBrand(BaseSapBrand bsb) {

        //校验入参 code不能重复
        QueryWrapper<BaseSapBrand> qw = new QueryWrapper<>();
        qw.eq("brand_code", bsb.getBrandCode());
        if (bsb.getId() != null) {
            qw.ne("id", bsb.getId());
        }
        List<BaseSapBrand> sapBrandList = baseSapBrandMapper.selectList(qw);
        if (CollectionUtil.isNotEmpty(sapBrandList)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "品牌 ID已存在");
        }

        if (bsb.getId() != null) {
            BaseSapBrand baseSapBrand = baseSapBrandMapper.selectById(bsb.getId());
            AssertUtil.isNull(baseSapBrand, "sap品牌不存在");
        }

        //新增or修改
        if (bsb.getId() != null) {
            baseSapBrandMapper.updateById(bsb);
        } else {
            baseSapBrandMapper.insert(bsb);
        }

    }
}
