package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.BaseSapCoverFactory;
import com.abi.qms.platform.dao.mapper.BaseSapCoverFactoryMapper;
import com.abi.qms.platform.dto.req.BatchImportCoverFactoryReq;
import com.abi.qms.platform.service.BaseSapCoverFactoryService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/04 15:23
 * @description:
 */
@Service
public class BaseSapCoverFactoryServiceImpl implements BaseSapCoverFactoryService {

    @Autowired
    private BaseSapCoverFactoryMapper baseSapCoverFactoryMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchImportCoverFactory(BatchImportCoverFactoryReq req) {

        List<BatchImportCoverFactoryReq.BatchImportCoverFactory> sapCoverFactoryList = req.getBatchImportCoverFactoryList();

        for (BatchImportCoverFactoryReq.BatchImportCoverFactory coverFactory : sapCoverFactoryList) {

            BaseSapCoverFactory baseSapCoverFactory = new BaseSapCoverFactory();
            baseSapCoverFactory.setSapCoverFactoryId(coverFactory.getSapCoverFactoryId());
            baseSapCoverFactory.setSapCoverFactoryName(coverFactory.getSapCoverFactoryName());
            baseSapCoverFactory.setKeymanName(coverFactory.getKeymanName());
            baseSapCoverFactory.setKeymanMobile(coverFactory.getKeymanMobile());

            saveBaseSapCoverFactory(baseSapCoverFactory);
        }

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdateCoverFactory(BatchImportCoverFactoryReq req) {

        List<BatchImportCoverFactoryReq.BatchImportCoverFactory> sapCoverFactoryList = req.getBatchImportCoverFactoryList();

        for (BatchImportCoverFactoryReq.BatchImportCoverFactory coverFactory : sapCoverFactoryList) {

            BaseSapCoverFactory baseSapCoverFactory = new BaseSapCoverFactory();
            baseSapCoverFactory.setSapCoverFactoryId(coverFactory.getSapCoverFactoryId());
            baseSapCoverFactory.setSapCoverFactoryName(coverFactory.getSapCoverFactoryName());
            baseSapCoverFactory.setKeymanName(coverFactory.getKeymanName());
            baseSapCoverFactory.setKeymanMobile(coverFactory.getKeymanMobile());

            updateBaseSapCoverFactory(baseSapCoverFactory);
        }

    }

    /**
     * 更新Sap供应商信息
     *
     * @param sapCoverFactory
     */
    private void updateBaseSapCoverFactory(BaseSapCoverFactory sapCoverFactory) {

        //根据sap供应商ID查询供应商信息
        BaseSapCoverFactory baseSapCoverFactory = getBaseSapCoverFactoryInfo(sapCoverFactory.getSapCoverFactoryId());

        if (ObjectUtil.isNull(baseSapCoverFactory)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap供应商ID不存在");
        }

        sapCoverFactory.setId(baseSapCoverFactory.getId());
        sapCoverFactory.setUpdateTime(LocalDateTime.now());
        baseSapCoverFactoryMapper.updateById(sapCoverFactory);

    }


    /**
     * 保存Sap供应商信息
     *
     * @param sapCoverFactory
     */
    private void saveBaseSapCoverFactory(BaseSapCoverFactory sapCoverFactory) {

        //根据sap供应商ID查询供应商信息
        BaseSapCoverFactory baseSapCoverFactory = getBaseSapCoverFactoryInfo(sapCoverFactory.getSapCoverFactoryId());

        if (ObjectUtil.isNotNull(baseSapCoverFactory)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap供应商ID已存在");
        }

        baseSapCoverFactoryMapper.insert(sapCoverFactory);
    }


    /**
     * 根据sap供应商ID查询sap供应商信息
     *
     * @param sapCoverFactoryId sap供应商ID
     * @return
     */
    private BaseSapCoverFactory getBaseSapCoverFactoryInfo(String sapCoverFactoryId) {

        QueryWrapper<BaseSapCoverFactory> qw = new QueryWrapper<>();
        qw.eq("sap_cover_factory_id", sapCoverFactoryId);
        qw.eq("is_delete", 0);

        //根据sap供应商id查询供应商信息
        BaseSapCoverFactory baseSapCoverFactory = baseSapCoverFactoryMapper.selectOne(qw);

        return baseSapCoverFactory;

    }


}
