package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.BaseSapMaterial;
import com.abi.qms.platform.dao.mapper.BaseSapMaterialMapper;
import com.abi.qms.platform.dto.req.BatchImportMaterialReq;
import com.abi.qms.platform.service.BaseSapMaterialService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/04 11:40
 * @description:
 */
@Service
public class BaseSapMaterialServiceImpl implements BaseSapMaterialService {

    @Autowired
    private BaseSapMaterialMapper baseSapMaterialMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchImportMaterial(BatchImportMaterialReq req) {
        List<BatchImportMaterialReq.BatchImportMaterial> sapMaterialList = req.getBatchImportMaterialList();

        for (BatchImportMaterialReq.BatchImportMaterial material : sapMaterialList) {

            BaseSapMaterial baseSapMaterial = new BaseSapMaterial();
            baseSapMaterial.setSapMaterialId(material.getSapMaterialId());
            baseSapMaterial.setSapMaterialName(material.getSapMaterialName());

            saveBaseSapMaterial(baseSapMaterial);
        }

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdateMaterial(BatchImportMaterialReq req) {
        List<BatchImportMaterialReq.BatchImportMaterial> sapMaterialList = req.getBatchImportMaterialList();

        for (BatchImportMaterialReq.BatchImportMaterial material : sapMaterialList) {

            BaseSapMaterial baseSapMaterial = new BaseSapMaterial();
            baseSapMaterial.setSapMaterialId(material.getSapMaterialId());
            baseSapMaterial.setSapMaterialName(material.getSapMaterialName());

            updateBaseSapMaterial(baseSapMaterial);
        }

    }

    /**
     * 更新Sap物料信息
     *
     * @param sapMaterial
     */
    private void updateBaseSapMaterial(BaseSapMaterial sapMaterial) {

        //根据sap物料ID查询物料信息
        BaseSapMaterial baseSapMaterial = getBaseSapMaterialInfo(sapMaterial.getSapMaterialId());

        if (ObjectUtil.isNull(baseSapMaterial)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap物料ID不存在");
        }

        sapMaterial.setId(baseSapMaterial.getId());
        sapMaterial.setUpdateTime(LocalDateTime.now());
        baseSapMaterialMapper.updateById(sapMaterial);

    }


    /**
     * 保存Sap物料信息
     *
     * @param sapMaterial
     */
    private void saveBaseSapMaterial(BaseSapMaterial sapMaterial) {

        //根据sap物料ID查询物料信息
        BaseSapMaterial baseSapMaterial = getBaseSapMaterialInfo(sapMaterial.getSapMaterialId());

        if (ObjectUtil.isNotNull(baseSapMaterial)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap物料ID已存在");
        }

        baseSapMaterialMapper.insert(sapMaterial);
    }


    private BaseSapMaterial getBaseSapMaterialInfo(String sapMaterialId) {

        QueryWrapper<BaseSapMaterial> qw = new QueryWrapper<>();
        qw.eq("sap_material_id", sapMaterialId);
        qw.eq("is_delete", 0);

        //根据sap物料id查询物料信息
        BaseSapMaterial baseSapMaterial = baseSapMaterialMapper.selectOne(qw);

        return baseSapMaterial;

    }


}
