package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.QrBoxCodeFormat;
import com.abi.qms.platform.dao.entity.QrBoxCodeFormatSplit;
import com.abi.qms.platform.dao.enums.FactoryTypeEnum;
import com.abi.qms.platform.dao.enums.StateEnum;
import com.abi.qms.platform.dao.mapper.QrBoxCodeFormatMapper;
import com.abi.qms.platform.dao.mapper.QrBoxCodeFormatSplitMapper;
import com.abi.qms.platform.dao.vo.result.QrBoxCodeFormatVO;
import com.abi.qms.platform.dao.vo.result.QrRepertorySelectDetailVO;
import com.abi.qms.platform.dto.req.DeleteBoxCodeFormatreq;
import com.abi.qms.platform.dto.req.DisableBoxCodeFormatReq;
import com.abi.qms.platform.dto.req.EnableBoxCodeFormatReq;
import com.abi.qms.platform.dto.req.ListBoxCodeFormatReq;
import com.abi.qms.platform.dto.req.QrBoxCodeFormatDetailReq;
import com.abi.qms.platform.dto.req.SaveBoxCodeFormatReq;
import com.abi.qms.platform.dto.res.ListBoxCodeFormatRes;
import com.abi.qms.platform.dto.res.QrBoxCodeFormatDetailRes;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.QrBoxCodeFormatService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/04/20 09:49
 * @description: 箱码幅面管理
 */
@Service
@Slf4j
public class QrBoxCodeFormatImpl implements QrBoxCodeFormatService {

    @Autowired
    private QrBoxCodeFormatMapper qrBoxCodeFormatMapper;

    @Autowired
    private QrBoxCodeFormatSplitMapper qrBoxCodeFormatSplitMapper;

    @Autowired
    private UserUtil userUtil;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveBoxCodeFormat(SaveBoxCodeFormatReq req) {

        //1-声明对象
        QrBoxCodeFormat qrBoxCodeFormat = null;
        if (req.getId() != null) {
            qrBoxCodeFormat = qrBoxCodeFormatMapper.selectById(req.getId());
            AssertUtil.isNull(qrBoxCodeFormat, "箱码幅面ID不存在");
        } else {
            qrBoxCodeFormat = new QrBoxCodeFormat();
        }

        //2-箱码幅面放入数据
        qrBoxCodeFormat.setQrRepertoryId(req.getQrRepertoryId());
        //若当前登录用户为包材厂用户，则该选项默认选择该用户所在的包材厂，不可修改
        qrBoxCodeFormat.setFactoryId(userUtil.getWholeUser().getFactoryId() != null ? userUtil.getWholeUser().getFactoryId() : req.getFactoryId());
        qrBoxCodeFormat.setName(req.getName());
        qrBoxCodeFormat.setColorPerRollScalar(req.getColorPerRollScalar());
        qrBoxCodeFormat.setFormatNumber(req.getFormatNumber());
        qrBoxCodeFormat.setRemark(req.getRemark());

        //3-新增or修改
        if (req.getId() != null) {
            qrBoxCodeFormatMapper.updateById(qrBoxCodeFormat);
        } else {
            qrBoxCodeFormatMapper.insert(qrBoxCodeFormat);
        }

        //4-新增or修改 拆分数量列表
        saveOrUpdateSplit(req, qrBoxCodeFormat.getId());

    }


    private void saveOrUpdateSplit(SaveBoxCodeFormatReq req, Long boxCodeFormatId) {

        List<SaveBoxCodeFormatReq.SplitBean> splitBeanList = req.getSplitBeanList();
        for (SaveBoxCodeFormatReq.SplitBean splitReq : splitBeanList) {
            //声明对象
            QrBoxCodeFormatSplit split = null;
            if (splitReq.getId() != null) {
                split = qrBoxCodeFormatSplitMapper.selectById(splitReq.getId());
                AssertUtil.isNull(split, "拆分数量ID不存在");
            } else {
                split = new QrBoxCodeFormatSplit();
            }
            //组装数据
            split.setBoxCodeFormatId(boxCodeFormatId);
            split.setQrRepertoryColumnId(splitReq.getQrRepertoryColumnId());
            split.setSplitNum(splitReq.getSplitNum());
            split.setQrRepertoryId(splitReq.getRepertoryId());

            if (splitReq.getId() != null) {
                //先删除再新增
                qrBoxCodeFormatSplitMapper.deleteBySplitId(splitReq.getId());
                qrBoxCodeFormatSplitMapper.insert(split);
            } else {
                qrBoxCodeFormatSplitMapper.insert(split);
            }
        }

    }

    @Override
    public ListBoxCodeFormatRes listBoxCodeFormat(ListBoxCodeFormatReq req) {
        //数据权限隔离，如果登录账号是包材厂，只能看账号包材厂的数据。否则看全部
        if (userUtil.getWholeUser().getFactoryId() != null && userUtil.getWholeUser().getFactoryType() != null && FactoryTypeEnum.COVER.is(userUtil.getWholeUser().getFactoryType())) {
            req.setFactoryId(userUtil.getWholeUser().getFactoryId());
        }
        //分页查询
        IPage<QrBoxCodeFormatVO> iPage = qrBoxCodeFormatMapper.listBoxCodeFormat(PageUtil.createPage(req), req);
        List<QrBoxCodeFormatVO> formatList = iPage.getRecords();
        //封装出参
        ListBoxCodeFormatRes res = new ListBoxCodeFormatRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListBoxCodeFormatRes.BoxCodeFormatBean> beanList = PojoConverterUtils.copyList(formatList, ListBoxCodeFormatRes.BoxCodeFormatBean.class);
        res.setBoxCodeFormatBeanList(beanList);

        return res;
    }

    @Override
    public QrBoxCodeFormatDetailRes getBoxCodeFormatDetail(QrBoxCodeFormatDetailReq req) {

        //查询箱码幅面详情
        QrBoxCodeFormatVO qrBoxCodeFormatVO = qrBoxCodeFormatMapper.selectBoxCodeFormatDetailById(req.getId());
        AssertUtil.isNull(qrBoxCodeFormatVO, "箱码幅面详情不存在");
        //查询箱码幅面下的拆分详情列表
        List<QrRepertorySelectDetailVO> voList = qrBoxCodeFormatSplitMapper.listSplitDetail(req.getId());

        //转化出参
        QrBoxCodeFormatDetailRes res = PojoConverterUtils.copy(qrBoxCodeFormatVO, QrBoxCodeFormatDetailRes.class);
        res.setSplitDetailList(voList);

        return res;
    }

    @Override
    public void enableBoxCodeFormat(EnableBoxCodeFormatReq req) {

        List<Long> ids = req.getIds();

        //循环启用
        for (Long id : ids) {
            //1-查询活动对象
            QrBoxCodeFormat qrBoxCodeFormat = qrBoxCodeFormatMapper.selectById(id);
            AssertUtil.isNull(qrBoxCodeFormat, "箱码幅面不存在");
            if (!StateEnum.DISABLE.is(qrBoxCodeFormat.getState())) {
                continue;
            }

            //2-修改状态
            qrBoxCodeFormat.setState(StateEnum.ENABLE.getCode());
            qrBoxCodeFormatMapper.updateById(qrBoxCodeFormat);
        }
    }

    @Override
    public void disableBoxCodeFormat(DisableBoxCodeFormatReq req) {

        List<Long> ids = req.getIds();

        //循环禁用
        for (Long id : ids) {
            //1-查询活动对象
            QrBoxCodeFormat qrBoxCodeFormat = qrBoxCodeFormatMapper.selectById(id);
            AssertUtil.isNull(qrBoxCodeFormat, "箱码幅面不存在");
            if (!StateEnum.ENABLE.is(qrBoxCodeFormat.getState())) {
                continue;
            }

            //2-修改状态
            qrBoxCodeFormat.setState(StateEnum.DISABLE.getCode());
            qrBoxCodeFormatMapper.updateById(qrBoxCodeFormat);
        }

    }

    @Override
    public void deleteBoxCodeFormat(DeleteBoxCodeFormatreq req) {

        //查询箱码幅面
        QrBoxCodeFormat qrBoxCodeFormat = qrBoxCodeFormatMapper.selectById(req.getId());
        if (qrBoxCodeFormat == null) {
            throw new BusinessException(ErrorCodeEnum.NO_EXISTS_DATA.getCode(), "箱码幅面不存在");
        }
        if (qrBoxCodeFormat.getState().equals(StateEnum.ENABLE.getCode())) {
            throw new BusinessException("该箱码幅面还在启用中，不能删除。");
        }

        //逻辑删除
        qrBoxCodeFormat.setIsDelete(1);
        qrBoxCodeFormatMapper.updateById(qrBoxCodeFormat);
        //根据箱码幅面ID逻辑删除码库拆分数据
        qrBoxCodeFormatSplitMapper.deleteByBoxCodeFormatId(req.getId());
    }


}
