package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.QrPackageSapOrderItem;
import com.abi.qms.platform.dao.mapper.QrPackageSapOrderItemMapper;
import com.abi.qms.platform.dto.req.BatchImportPackageSapOrderItemReq;
import com.abi.qms.platform.service.QrPackageSapOrderItemService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/07 09:30
 * @description:
 */
@Service
public class QrPackageSapOrderItemServiceImpl implements QrPackageSapOrderItemService {

    @Autowired
    private QrPackageSapOrderItemMapper qrPackageSapOrderItemMapper;

    @Override
    public void batchImportPackageSapOrderItem(BatchImportPackageSapOrderItemReq req) {

        List<BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem> sapOrderItemList = req.getBatchImportSapOrderItemList();
        for (BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem item : sapOrderItemList) {

            QrPackageSapOrderItem sapOrderItem = new QrPackageSapOrderItem();
            sapOrderItem.setSapOrderNo(item.getSapOrderNo());
            sapOrderItem.setSapMaterialId(item.getSapMaterialId());
            sapOrderItem.setSapCoverFactoryId(item.getSapCoverFactoryId());
            sapOrderItem.setSapBeerFactoryId(item.getSapBeerFactoryId());
            sapOrderItem.setSapMaterialName(item.getSapMaterialName());
            sapOrderItem.setPurchaseCount(item.getPurchaseCount());

            saveQrPackageSapOrderItem(sapOrderItem);
        }

    }

    @Override
    public void batchUpdatePackageSapOrderItem(BatchImportPackageSapOrderItemReq req) {

        List<BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem> sapOrderItemList = req.getBatchImportSapOrderItemList();
        for (BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem item : sapOrderItemList) {

            QrPackageSapOrderItem sapOrderItem = new QrPackageSapOrderItem();
            sapOrderItem.setSapOrderNo(item.getSapOrderNo());
            sapOrderItem.setSapMaterialId(item.getSapMaterialId());
            sapOrderItem.setSapCoverFactoryId(item.getSapCoverFactoryId());
            sapOrderItem.setSapBeerFactoryId(item.getSapBeerFactoryId());
            sapOrderItem.setSapMaterialName(item.getSapMaterialName());
            sapOrderItem.setPurchaseCount(item.getPurchaseCount());

            updateQrPackageSapOrderItem(sapOrderItem);
        }

    }

    private void updateQrPackageSapOrderItem(QrPackageSapOrderItem sapOrderItem) {

        //根据sap订单号查询物料订单明细信息
        QrPackageSapOrderItem qrPackageSapOrderItem = getQrPackageSapOrderItemInfo(sapOrderItem.getSapOrderNo());

        if (ObjectUtil.isNull(qrPackageSapOrderItem)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap物料订单明细订单号不存在");
        }

        sapOrderItem.setId(qrPackageSapOrderItem.getId());
        sapOrderItem.setUpdateTime(LocalDateTime.now());
        qrPackageSapOrderItemMapper.updateById(sapOrderItem);
    }

    private void saveQrPackageSapOrderItem(QrPackageSapOrderItem sapOrderItem) {

        //根据sap订单号查询物料订单明细信息
        QrPackageSapOrderItem qrPackageSapOrderItem = getQrPackageSapOrderItemInfo(sapOrderItem.getSapOrderNo());

        if (ObjectUtil.isNotNull(qrPackageSapOrderItem)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap物料订单明细订单号已存在");
        }

        qrPackageSapOrderItemMapper.insert(sapOrderItem);
    }

    private QrPackageSapOrderItem getQrPackageSapOrderItemInfo(String sapOrderNo) {

        QueryWrapper<QrPackageSapOrderItem> qw = new QueryWrapper<>();
        qw.eq("sap_order_no", sapOrderNo);
        qw.eq("is_delete", 0);

        //根据sap订单号查询物料订单明细信息
        QrPackageSapOrderItem qrPackageSapOrderItem = qrPackageSapOrderItemMapper.selectOne(qw);

        return qrPackageSapOrderItem;

    }

}
