package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.QrPackageSapOrder;
import com.abi.qms.platform.dao.mapper.QrPackageSapOrderMapper;
import com.abi.qms.platform.dto.req.BatchImportPackageSapOrderReq;
import com.abi.qms.platform.service.QrPackageSapOrderService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/04 17:40
 * @description:
 */
@Service
public class QrPackageSapOrderServiceImpl implements QrPackageSapOrderService {

    @Autowired
    private QrPackageSapOrderMapper qrPackageSapOrderMapper;

    @Override
    public void batchImportPackageSapOrder(BatchImportPackageSapOrderReq req) {

        List<BatchImportPackageSapOrderReq.BatchImportSapOrder> sapOrderList = req.getBatchImportSapOrderList();
        for (BatchImportPackageSapOrderReq.BatchImportSapOrder order : sapOrderList) {
            QrPackageSapOrder sapOrder = new QrPackageSapOrder();
            sapOrder.setSapOrderCommitTime(order.getSapOrderCommitTime());
            sapOrder.setSapOrderNo(order.getSapOrderNo());

            saveQrPackageSapOrder(sapOrder);
        }

    }

    @Override
    public void batchUpdatePackageSapOrder(BatchImportPackageSapOrderReq req) {

        List<BatchImportPackageSapOrderReq.BatchImportSapOrder> sapOrderList = req.getBatchImportSapOrderList();
        for (BatchImportPackageSapOrderReq.BatchImportSapOrder order : sapOrderList) {
            QrPackageSapOrder sapOrder = new QrPackageSapOrder();
            sapOrder.setSapOrderCommitTime(order.getSapOrderCommitTime());
            sapOrder.setSapOrderNo(order.getSapOrderNo());

            updateQrPackageSapOrder(sapOrder);
        }

    }

    private void updateQrPackageSapOrder(QrPackageSapOrder sapOrder) {

        //根据sap订单号查询订单信息
        QrPackageSapOrder qrPackageSapOrder = getQrPackageSapOrderInfo(sapOrder.getSapOrderNo());

        if (ObjectUtil.isNull(qrPackageSapOrder)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap订单号不存在");
        }

        sapOrder.setId(qrPackageSapOrder.getId());
        sapOrder.setUpdateTime(LocalDateTime.now());
        qrPackageSapOrderMapper.updateById(sapOrder);
    }

    private void saveQrPackageSapOrder(QrPackageSapOrder sapOrder) {

        //根据sap订单号查询订单信息
        QrPackageSapOrder qrPackageSapOrder = getQrPackageSapOrderInfo(sapOrder.getSapOrderNo());

        if (ObjectUtil.isNotNull(qrPackageSapOrder)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "Sap订单号已存在");
        }

        qrPackageSapOrderMapper.insert(sapOrder);
    }

    private QrPackageSapOrder getQrPackageSapOrderInfo(String sapOrderNo) {

        QueryWrapper<QrPackageSapOrder> qw = new QueryWrapper<>();
        qw.eq("sap_order_no", sapOrderNo);
        qw.eq("is_delete", 0);

        //根据sap订单号查询码包订单信息
        QrPackageSapOrder qrPackageSapOrder = qrPackageSapOrderMapper.selectOne(qw);

        return qrPackageSapOrder;

    }
}
