package com.abi.qms.platform;

import com.abi.qms.platform.dao.tablestore.entity.QrCode;
import com.abi.task.common.tablestore.TableStorePlusUtils;
import com.abi.task.common.tablestore.common.TableStoreReq;
import com.abi.task.common.tablestore.common.TableStoreRes;
import com.abi.task.common.tablestore.entity.DemoEntity;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

@SpringBootTest
@Slf4j
public class TableStorePlusTest {

    @Autowired
    private TableStorePlusUtils tableStorePlusUtils;

    @Test
    public void testAddRow() {
        DemoEntity entity = new DemoEntity();
        entity.setQrCode("10293827182717");
        entity.setPayAmount(990.12);
        entity.setCurrentTime(LocalDateTime.now());
        entity.setUserId(10001L);
        entity.setActiveCount(33);
        entity.setIsDog(false);

        DemoEntity entity2 = new DemoEntity();
        entity2.setQrCode("10293827182716");
        entity2.setPayAmount(990.12);
        entity2.setCurrentTime(LocalDateTime.now());
        entity2.setUserId(10001L);
        entity2.setActiveCount(33);
        entity2.setIsDog(false);

        tableStorePlusUtils.putRow(Arrays.asList(entity, entity2));
    }

    @Test
    public void testGetRow() {
        QrCode entity = tableStorePlusUtils.selectOne(QrCode.class, "1397521498780426242");
        Long codeIndex = entity.getCodeIndex();
        log.info("======" + codeIndex);
    }

    //按索引范围匹配
    @Test
    public void queryRowsRange(){
        TableStoreReq<QrCode> req = TableStoreReq.build(QrCode.class,"codeIndex")
                .range(1,100)
                .page(1,15)
                .isDesc();
        //调用查询
        TableStoreRes<QrCode> res = tableStorePlusUtils.selectList(req);

        List<QrCode> qrCodeList = res.getEntityList();

        //打印
        for(QrCode qrCode:qrCodeList){
            log.info(qrCode.toString());
        }
    }

    //按索引精确匹配
    @Test
    public void queryRowsEqual(){
        TableStoreReq<QrCode> req = TableStoreReq.build(QrCode.class,"packageId")
                .eq("1")
                .page(1,3)
                .isNeedTotalCount();

        TableStoreRes<QrCode> res = tableStorePlusUtils.selectList(req);

        List<QrCode> qrCodeList = res.getEntityList();

        //打印
        for(QrCode qrCode:qrCodeList){
            log.info(qrCode.toString());
        }
        log.info("total count----------->");
        log.info(String.valueOf(res.getTotalCount()));
    }



    @Test
    public void testDeleteRow() {
        tableStorePlusUtils.deleteRow(DemoEntity.class, "10293827182717");
    }



}
